#  Copyright 2015-2016 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
"""Unittest of handling for user session."""

import unittest
from unittest import mock

from yanico import session
from yanico.session import LoaderNotFoundError


class TestLoad(unittest.TestCase):
    """Test for yanico.session.load()."""

    @mock.patch("yanico.session.entry_points")
    def test_loader_exist(self, entry_points):
        """Expect to the load function returns user session."""
        entry = mock.Mock()
        loader = entry.load.return_value = mock.Mock(return_value="value")
        entry_points.return_value = [entry]

        ltype = "firefox"
        profile = "/path/to/profile"
        self.assertEqual(session.load(ltype, profile), "value")

        entry_points.assert_called_once_with(group="yanico.sessions", name=ltype)
        entry.load.assert_called_once_with()
        loader.assert_called_once_with(profile)

    @mock.patch("yanico.session.entry_points")
    def test_loader_not_exist(self, entry_points):
        """Expect to raise exception when the loader does not exist."""
        eps = entry_points.return_value
        eps.return_value = []

        ltype = "loader is not found"
        profile = "dummy"
        self.assertRaises(LoaderNotFoundError, session.load, ltype, profile)

        entry_points.assert_called_once_with(group="yanico.sessions", name=ltype)

    @mock.patch("yanico.session.entry_points")
    def test_loader_error(self, entry_points):
        """Expect to through error when the loader raises any error."""

        class _AnyError(Exception):
            pass

        entry = mock.Mock()
        loader = entry.load.return_value = mock.Mock(side_effect=_AnyError)
        entry_points.return_value = [entry]

        ltype = "loader is found"
        profile = "a profile"
        self.assertRaises(_AnyError, session.load, ltype, profile)

        entry_points.assert_called_once_with(group="yanico.sessions", name=ltype)
        entry.load.assert_called_once_with()
        loader.assert_called_once_with(profile)


class TestLoadFromConfig(unittest.TestCase):
    """Test for yanico.session.load_from_config()."""

    @mock.patch("yanico.session.load")
    def test_min_config(self, load_func):
        """Expect to return string using minimum configuration."""
        load_func.return_value = "value"

        min_config = {"session": {"type": "foobar", "profile": "/path/to/profile"}}
        self.assertEqual(session.load_from_config(min_config), "value")

        load_func.assert_called_once_with("foobar", "/path/to/profile")

    def test_without_session(self):
        """Expect to raise KeyError if the config without session."""
        without_session = {"type": "foobar", "profile": "/path/to/profile"}
        self.assertRaises(KeyError, session.load_from_config, without_session)

    def test_without_type(self):
        """Expect to raise KeyError if the config without type."""
        without_type = {"session": {"profile": "/path/to/profile"}}
        self.assertRaises(KeyError, session.load_from_config, without_type)

    def test_without_profile(self):
        """Expect to raise KeyError if the config without profile."""
        without_profile = {"session": {"type": "foobar"}}
        self.assertRaises(KeyError, session.load_from_config, without_profile)



#  Copyright 2015-2016 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
"""Unittest of configuration loading."""

import os
import unittest
from unittest import mock

from yanico import config


class TestUserPath(unittest.TestCase):
    """Test for yanico.config.user_path()."""

    @mock.patch.dict(os.environ, {"HOME": "spam"})
    def test_path(self):
        """Expect filepath joinning '.yanico.conf' under $HOME."""
        expect = None
        if os.sep == "\\":
            expect = "spam\\.yanico.conf"
        elif os.sep == "/":
            expect = "spam/.yanico.conf"
        result = config.user_path()
        self.assertEqual(result, expect)

    @mock.patch("yanico.config.CONFIG_FILENAME", new="ham.egg")
    def test_dependence_constants(self):
        """Expect to depend filename by 'CONFIG_FILENAME' constants."""
        result = config.user_path()
        self.assertEqual(os.path.basename(result), "ham.egg")


class TestLoad(unittest.TestCase):
    """Test for yanico.config.load()."""

    @mock.patch("configparser.ConfigParser")
    def test_without_args(self, parserclass):
        """Expect config object that tried to parse user config file."""
        mock_conf = parserclass.return_value
        result = config.load()
        self.assertIs(result, mock_conf)
        mock_conf.read.assert_called_once_with((config.user_path(),))

    @mock.patch("configparser.ConfigParser")
    def test_with_args(self, parserclass):
        """Expect arguments pass to the parser after user config file."""
        mock_conf = parserclass.return_value
        result = config.load("spam", "ham", "eggs")
        self.assertIs(result, mock_conf)
        mock_conf.read.assert_called_once_with(
            (config.user_path(), "spam", "ham", "eggs")
        )



"""Command entry point."""

#  Copyright 2015 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import argparse
import sys

import yanico

if sys.version_info < (3, 10):
    from importlib_metadata import entry_points
else:
    from importlib.metadata import entry_points


def create_main_parser():
    """Return command argument parser."""
    parser = argparse.ArgumentParser(
        description=yanico.__description__,
        prog="yan",
    )
    parser.add_argument(
        "--version", action="version", version="yanico version " + yanico.__version__
    )
    parser.set_defaults(run=lambda _: parser.print_help())
    return parser


def build_subparsers(parser):
    """Register command from setuptools plugins."""
    # pylint: disable=no-member
    subparsers = parser.add_subparsers(title="Commands")
    eps = entry_points(group="yanico.commands")
    for entry in eps:
        register = entry.load()
        register(entry.name, subparsers)


def main():
    """Command entry point."""
    parser = create_main_parser()
    build_subparsers(parser)
    args = parser.parse_args()
    return args.run(args)



#  Copyright 2015-2016 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
"""Handle nicovideo.jp user_session."""

import sys

if sys.version_info < (3, 10):
    from importlib_metadata import entry_points
else:
    from importlib.metadata import entry_points


class LoaderNotFoundError(Exception):
    """Session loader is not found."""


class UserSessionNotFoundError(Exception):
    """Profile exists, but user_session is not found."""


def load(ltype, profile):
    """Return nicovideo.jp user session string.

    Args:
        ltype (str): loader type
        profile (str): file path for profile

    Returns:
        str: user session

    Raises:
        LoaderNotFoundError
        Error from loader
    """
    eps = entry_points(group="yanico.sessions", name=ltype)
    for entry in eps:
        load_func = entry.load()
        return load_func(profile)
    raise LoaderNotFoundError(f"{ltype} loader is not found.")


def load_from_config(conf):
    """Return nicovideo.jp user session string.

    Args:
        conf (ConfigMap): configuration of the user session loading
                          ConfigMap = Mapping[str, Mapping[str, str]]

    Returns:
        str: user session

    Raises:
        KeyError
        Error from load()
    """
    session = conf["session"]
    return load(session["type"], session["profile"])



"""Handle Firefox session."""

#  Copyright 2015-2016 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os.path
import sqlite3

from yanico.session import UserSessionNotFoundError


def load(profile):
    """Return the nicovideo.jp user_session string.

    Arguments:
    profile : str -- path for Firefox profile direcotry

    Returns: str

    Raises:
    FileNotFoundError
    UserSessionNotFoundError
    """
    if not profile:
        raise FileNotFoundError("profile must not be empty.")
    cookie_path = os.path.join(profile, "cookies.sqlite")
    try:
        conn = sqlite3.connect(cookie_path)
    except sqlite3.OperationalError as exc:
        raise FileNotFoundError(f"{profile} is not directory.") from exc
    cur = conn.execute(
        "SELECT value FROM moz_cookies "
        "WHERE name = 'user_session' AND host = '.nicovideo.jp'"
    )
    row = cur.fetchone()
    if not row:
        raise UserSessionNotFoundError("user_session is not found.")
    return row[0]



"""Yet Another Niconico-douga Command-line Interface."""

#  Copyright 2015-2024 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from importlib.metadata import PackageNotFoundError, version

__author__ = "Masayuki Yamamoto"
__copyright__ = "Copyright 2015-2024 Yamamoto Masayuki"
__description__ = __doc__
__license__ = "Apache 2.0"

try:
    __version__ = version(__name__)
except PackageNotFoundError:
    __version__ = "unknown"



"""Unittest of firefox session."""

#  Copyright 2015-2016 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os.path
import sqlite3
import sys
import unittest
from unittest import mock

from yanico.session import firefox
from yanico.session import UserSessionNotFoundError

if sys.version_info < (3, 10):
    from importlib_metadata import entry_points
else:
    from importlib.metadata import entry_points


def _stub_db():
    conn = sqlite3.connect(":memory:")
    conn.execute("CREATE TABLE moz_cookies (name, host, value)")
    return conn


class TestLoad(unittest.TestCase):
    """Test for yanico.session.firefox.load()."""

    def test_exist(self):
        """Expect user_session string."""
        conn = _stub_db()
        conn.execute(
            "INSERT INTO moz_cookies VALUES"
            "('user_session', '.nicovideo.jp', 'dummy_user_session')"
        )
        with mock.patch("sqlite3.connect") as mock_connect:
            mock_connect.return_value = conn
            ses = firefox.load("/path/to/profile")
        expect_path = os.path.join("/path/to/profile", "cookies.sqlite")
        mock_connect.assert_called_once_with(expect_path)
        self.assertEqual("dummy_user_session", ses)

    def test_not_exist(self):
        """If not exist user_session, raise Error."""
        conn = _stub_db()
        with mock.patch("sqlite3.connect") as mock_connect:
            mock_connect.return_value = conn
            self.assertRaises(
                UserSessionNotFoundError, firefox.load, "/path/to/profile"
            )
        expect_path = os.path.join("/path/to/profile", "cookies.sqlite")
        mock_connect.assert_called_once_with(expect_path)

    def test_not_directory(self):
        """If profile is not directory path, raise Error."""
        self.assertRaises(FileNotFoundError, firefox.load, "/NOT-DIRECTORY")

    def test_empty_string(self):
        """If profile is empty string, raise Error."""
        self.assertRaises(FileNotFoundError, firefox.load, "")


class TestEntryPoint(unittest.TestCase):
    """Test for `yanico.sessions` entry point."""

    def test_load_func(self):
        """Check whether loaeded function is firefox.load()."""
        (entry,) = entry_points(group="yanico.sessions", name="firefox")
        func = entry.load()
        self.assertIs(firefox.load, func)



"""Unittest of command entry point."""

#  Copyright 2015 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import argparse
import unittest
from unittest import mock

import yanico
import yanico.command


class TestCreateMainParser(unittest.TestCase):
    """yanico.command.create_main_parser() test."""

    def test_version(self):
        """Parse '--version' option."""
        parser = yanico.command.create_main_parser()
        with mock.patch.object(parser, "_print_message") as print_message:
            self.assertRaises(SystemExit, parser.parse_args, ["--version"])
        print_message.assert_called_once_with(
            "yanico version " + yanico.__version__ + "\n", mock.ANY
        )

    def test_help_long(self):
        """Parse '--help' option."""
        parser = yanico.command.create_main_parser()
        with mock.patch.object(parser, "print_help") as print_help:
            self.assertRaises(SystemExit, parser.parse_args, ["--help"])
        print_help.assert_called_once_with()

    def test_help_short(self):
        """Parse '-h' option."""
        parser = yanico.command.create_main_parser()
        with mock.patch.object(parser, "print_help") as print_help:
            self.assertRaises(SystemExit, parser.parse_args, ["-h"])
        print_help.assert_called_once_with()

    @staticmethod
    def test_without_argments():
        """Expect 'run' method that showing help."""
        parser = yanico.command.create_main_parser()
        args = parser.parse_args([])
        with mock.patch.object(parser, "print_help") as print_help:
            args.run(args)
        print_help.assert_called_once_with()


class TestBuildSubparsers(unittest.TestCase):
    """yanico.command.build_subparsers() test."""

    @staticmethod
    def _init_stub_command():
        """Return entry point having stub command."""

        def run(args):
            """Command body."""
            return args.n * 2

        def register(command_name, subparsers):
            """Command registrant."""
            parser = subparsers.add_parser(command_name)
            parser.add_argument("n", type=int)
            parser.set_defaults(run=run)

        entry_point = mock.Mock()
        entry_point.name = "stub"
        entry_point.load.return_value = register
        return entry_point

    @mock.patch("yanico.command.entry_points")
    def test_stub_command(self, entry_points):
        """Register command returning two times numbers."""
        entry_points.return_value = [self._init_stub_command()]

        parser = argparse.ArgumentParser()
        yanico.command.build_subparsers(parser)
        args = parser.parse_args(["stub", "21"])
        self.assertEqual(args.run(args), 42)
        entry_points.assert_called_once_with(group="yanico.commands")

    @mock.patch("yanico.command.entry_points")
    def test_without_arguments(self, entry_points):
        """Expect 'run' method that showing help."""
        entry_points.return_value = [self._init_stub_command()]

        parser = yanico.command.create_main_parser()
        yanico.command.build_subparsers(parser)
        args = parser.parse_args([])
        with mock.patch.object(parser, "print_help") as print_help:
            args.run(args)
        print_help.assert_called_once_with()
        entry_points.assert_called_once_with(group="yanico.commands")


class TestMain(unittest.TestCase):
    """yanico.command.main() test."""

    @staticmethod
    def _init_stub_parser():
        """Return parser having command that returns integer."""
        parser = argparse.ArgumentParser()
        subparsers = parser.add_subparsers()
        stub_parser = subparsers.add_parser("stub")
        stub_parser.add_argument("n", type=int)
        stub_parser.set_defaults(run=lambda args: args.n)
        return parser

    @mock.patch("yanico.command.create_main_parser")
    @mock.patch("yanico.command.build_subparsers")
    @mock.patch("sys.argv", new=["", "stub", "3"])
    def test_return_value(self, _, create_main_parser):
        """Except main function returns specified integer."""
        create_main_parser.return_value = self._init_stub_parser()
        self.assertEqual(yanico.command.main(), 3)



#  Copyright 2015-2016 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
"""Handle yanico configuration."""

import configparser
import os.path


CONFIG_FILENAME = ".yanico.conf"


def user_path():
    """Return user configuration filepath.

    The filepath depends home directory and CONFIG_FILENAME constants.
    """
    return os.path.join(os.path.expanduser("~"), CONFIG_FILENAME)


def load(*filepaths):
    """Return configration object.

    Object parses home directory config file.

    Args:
        filepaths (Tuple[str]): configuration file paths

    Returns:
        ConfigParser: object expects some configurations are loaded.
    """
    parser = configparser.ConfigParser()
    parser.read((user_path(),) + filepaths)
    return parser
